"""Tests for the Darwin client."""

import pytest

# Remove unused import
import pathlib
import time
import tempfile
import logging
from datetime import datetime
from collections import defaultdict
from unittest.mock import MagicMock, AsyncMock, patch
from bs4 import BeautifulSoup

from mcp_nixos.clients.darwin.darwin_client import DarwinClient, DarwinOption
from mcp_nixos.clients.html_client import HTMLClient
from mcp_nixos.cache.html_cache import HTMLCache
from mcp_nixos.cache.simple_cache import SimpleCache
from mcp_nixos.utils.cache_helpers import get_default_cache_dir


@pytest.fixture
def sample_html():
    """Return sample HTML for testing."""
    return """
    <html>
    <body>
        <dl>
            <dt>
                <a id="opt-system.defaults.dock.autohide"></a>
                <code>system.defaults.dock.autohide</code>
            </dt>
            <dd>
                Whether to automatically hide and show the dock. The default is false.
                *Type:* boolean
                *Default:* false
                *Example:* true
                *Declared by:* &lt;nix-darwin/modules/system/defaults.nix&gt;
            </dd>
            <dt>
                <a id="opt-system.defaults.dock.orientation"></a>
                <code>system.defaults.dock.orientation</code>
            </dt>
            <dd>
                Position of the dock on screen. The default is "bottom".
                *Type:* string
                *Default:* bottom
                *Example:* left
                *Declared by:* &lt;nix-darwin/modules/system/defaults.nix&gt;
            </dd>
        </dl>
    </body>
    </html>
    """


@pytest.fixture
def mock_html_cache():
    """Create a mock HTML cache."""
    cache = MagicMock(spec=HTMLCache)
    cache.get = AsyncMock(return_value=None)
    cache.set = AsyncMock(return_value=None)
    return cache


@pytest.fixture
def mock_html_client(sample_html, mock_html_cache):
    """Create a mock HTML client."""
    client = MagicMock(spec=HTMLClient)
    client.fetch = MagicMock(return_value=(sample_html, {"success": True}))
    # Add the cache attribute that our code now expects
    client.cache = mock_html_cache
    return client


@pytest.fixture
def mock_memory_cache():
    """Create a mock memory cache."""
    cache = MagicMock(spec=SimpleCache)
    # SimpleCache methods are synchronous, not async
    cache.get = MagicMock(return_value=None)
    cache.set = MagicMock(return_value=None)
    return cache


@pytest.fixture
def darwin_client(mock_html_client, mock_memory_cache):
    """Create a Darwin client for testing."""
    with patch("mcp_nixos.clients.darwin.darwin_client.SimpleCache", return_value=mock_memory_cache):
        # We no longer need to patch HTMLCache since we're reusing html_client.cache
        client = DarwinClient(html_client=mock_html_client)
        return client


@pytest.mark.asyncio
async def test_load_options(darwin_client, sample_html):
    """Test loading options from HTML."""
    # Mock the memory cache to return None to force HTML parsing
    darwin_client.memory_cache.get = MagicMock(return_value=None)

    # Call the method
    result = await darwin_client.load_options()

    # Check that options were loaded
    assert "system.defaults.dock.autohide" in result
    assert "system.defaults.dock.orientation" in result
    assert darwin_client.total_options == 2
    assert darwin_client.total_categories == 1
    assert darwin_client.loading_status == "loaded"


@pytest.mark.asyncio
async def test_parse_options(darwin_client, sample_html):
    """Test parsing options from HTML."""
    soup = BeautifulSoup(sample_html, "html.parser")
    await darwin_client._parse_options(soup)

    # Check that options were parsed correctly
    assert "system.defaults.dock.autohide" in darwin_client.options
    assert "system.defaults.dock.orientation" in darwin_client.options

    # Check option details
    option = darwin_client.options["system.defaults.dock.autohide"]
    assert option.name == "system.defaults.dock.autohide"
    assert "hide and show the dock" in option.description
    assert "boolean" in option.type
    assert "false" in option.default
    assert "true" in option.example

    # Check indices
    assert "system" in darwin_client.name_index
    assert "system.defaults" in darwin_client.name_index
    assert "system.defaults.dock" in darwin_client.name_index
    assert "system.defaults.dock.autohide" in darwin_client.name_index

    # Check word index
    assert "hide" in darwin_client.word_index
    assert "dock" in darwin_client.word_index
    assert len(darwin_client.word_index["dock"]) > 0


@pytest.mark.asyncio
async def test_search_options(darwin_client):
    """Test searching for options."""
    # Set up test data
    darwin_client.options = {
        "system.defaults.dock.autohide": DarwinOption(
            name="system.defaults.dock.autohide",
            description="Whether to automatically hide and show the dock.",
            type="boolean",
            default="false",
            example="true",
            declared_by="system/defaults.nix",
        ),
        "system.defaults.dock.orientation": DarwinOption(
            name="system.defaults.dock.orientation",
            description="Position of the dock on screen.",
            type="string",
            default="bottom",
            example="left",
            declared_by="system/defaults.nix",
        ),
    }
    darwin_client.name_index = {
        "system": ["system.defaults.dock.autohide", "system.defaults.dock.orientation"],
        "system.defaults": ["system.defaults.dock.autohide", "system.defaults.dock.orientation"],
        "system.defaults.dock": ["system.defaults.dock.autohide", "system.defaults.dock.orientation"],
        "system.defaults.dock.autohide": ["system.defaults.dock.autohide"],
        "system.defaults.dock.orientation": ["system.defaults.dock.orientation"],
    }
    darwin_client.word_index = {
        "dock": {"system.defaults.dock.autohide", "system.defaults.dock.orientation"},
        "hide": {"system.defaults.dock.autohide"},
        "orientation": {"system.defaults.dock.orientation"},
    }
    darwin_client.prefix_index = {
        "system": ["system.defaults.dock.autohide", "system.defaults.dock.orientation"],
        "system.defaults": ["system.defaults.dock.autohide", "system.defaults.dock.orientation"],
        "system.defaults.dock": ["system.defaults.dock.autohide", "system.defaults.dock.orientation"],
    }

    # Test exact match
    results = await darwin_client.search_options("system.defaults.dock.autohide")
    assert len(results) >= 1  # It may match both due to how search works
    assert any(r["name"] == "system.defaults.dock.autohide" for r in results)

    # Test prefix match
    results = await darwin_client.search_options("system.defaults.dock")
    assert len(results) == 2
    names = [r["name"] for r in results]
    assert "system.defaults.dock.autohide" in names
    assert "system.defaults.dock.orientation" in names

    # Test word match
    results = await darwin_client.search_options("hide")
    assert len(results) == 1
    assert results[0]["name"] == "system.defaults.dock.autohide"


@pytest.mark.asyncio
async def test_get_option(darwin_client):
    """Test getting a specific option."""
    # Set up test data
    darwin_client.options = {
        "system.defaults.dock.autohide": DarwinOption(
            name="system.defaults.dock.autohide",
            description="Whether to automatically hide and show the dock.",
            type="boolean",
            default="false",
            example="true",
            declared_by="system/defaults.nix",
        )
    }

    # Test getting an existing option
    option = await darwin_client.get_option("system.defaults.dock.autohide")
    assert option is not None
    assert option["name"] == "system.defaults.dock.autohide"
    assert "hide and show the dock" in option["description"]
    assert option["type"] == "boolean"

    # Test getting a non-existent option
    option = await darwin_client.get_option("nonexistent")
    assert option is None


@pytest.mark.asyncio
async def test_get_options_by_prefix(darwin_client):
    """Test getting options by prefix."""
    # Set up test data
    darwin_client.options = {
        "system.defaults.dock.autohide": DarwinOption(
            name="system.defaults.dock.autohide",
            description="Whether to automatically hide and show the dock.",
            type="boolean",
            default="false",
            example="true",
            declared_by="system/defaults.nix",
        ),
        "system.defaults.dock.orientation": DarwinOption(
            name="system.defaults.dock.orientation",
            description="Position of the dock on screen.",
            type="string",
            default="bottom",
            example="left",
            declared_by="system/defaults.nix",
        ),
        "system.keyboard.enableKeyMapping": DarwinOption(
            name="system.keyboard.enableKeyMapping",
            description="Whether to enable keyboard mapping.",
            type="boolean",
            default="false",
            example="true",
            declared_by="system/keyboard.nix",
        ),
    }
    darwin_client.prefix_index = {
        "system": [
            "system.defaults.dock.autohide",
            "system.defaults.dock.orientation",
            "system.keyboard.enableKeyMapping",
        ],
        "system.defaults": ["system.defaults.dock.autohide", "system.defaults.dock.orientation"],
        "system.defaults.dock": ["system.defaults.dock.autohide", "system.defaults.dock.orientation"],
        "system.keyboard": ["system.keyboard.enableKeyMapping"],
    }

    # Test getting options by prefix
    options = await darwin_client.get_options_by_prefix("system.defaults.dock")
    assert len(options) == 2
    names = [o["name"] for o in options]
    assert "system.defaults.dock.autohide" in names
    assert "system.defaults.dock.orientation" in names

    # Test getting options by a different prefix
    options = await darwin_client.get_options_by_prefix("system.keyboard")
    assert len(options) == 1
    assert options[0]["name"] == "system.keyboard.enableKeyMapping"


@pytest.mark.asyncio
async def test_fetch_url(darwin_client, sample_html):
    """Test the fetch_url adapter method."""
    # Setup the mock to return proper data
    darwin_client.html_client.fetch.return_value = (sample_html, {"success": True})

    # Test successful fetch
    result = await darwin_client.fetch_url("https://example.com")
    assert result == sample_html
    darwin_client.html_client.fetch.assert_called_once_with("https://example.com", force_refresh=False)

    # Test error handling
    darwin_client.html_client.fetch.reset_mock()
    darwin_client.html_client.fetch.return_value = (None, {"error": "Connection error"})

    with pytest.raises(ValueError) as excinfo:
        await darwin_client.fetch_url("https://example.com")

    assert "Connection error" in str(excinfo.value)


def test_cache_initialization():
    """Test that the DarwinClient correctly uses the HTMLClient's cache."""
    # Create a real HTMLClient with a proper cache
    html_client = HTMLClient()

    # Create a DarwinClient with the real client
    darwin_client = DarwinClient(html_client=html_client)

    # Verify that the html_cache is the same object as the html_client's cache
    assert darwin_client.html_cache is html_client.cache

    # Verify the cache directory is not "darwin" but the proper OS-specific path
    assert darwin_client.html_cache is not None, "HTML cache should not be None"
    assert darwin_client.html_cache.cache_dir != pathlib.Path("darwin")

    # Check that the cache directory is a subpath of the default cache dir
    default_cache_dir = pathlib.Path(get_default_cache_dir())
    assert darwin_client.html_cache is not None, "HTML cache should not be None"
    assert str(darwin_client.html_cache.cache_dir).startswith(str(default_cache_dir))

    # Create a darwin client without passing a client to test the default case
    with patch("mcp_nixos.clients.darwin.darwin_client.HTMLClient") as mock_html_client_class:
        # Setup the mock to return a client with a proper cache
        mock_client = MagicMock()
        mock_cache = MagicMock()
        mock_client.cache = mock_cache
        mock_html_client_class.return_value = mock_client

        # Create a new darwin client that will use our mocked HTMLClient
        client = DarwinClient()

        # Verify HTMLClient was created with the proper TTL
        mock_html_client_class.assert_called_once_with(ttl=client.cache_ttl)

        # Verify that the html_cache is the same as the client's cache
        assert client.html_cache is mock_cache


def test_avoid_read_only_filesystem_error():
    """Test that the DarwinClient doesn't try to create a 'darwin' directory in the current path."""
    # Create an actual client
    darwin_client = DarwinClient()

    # Make sure there is no 'darwin' directory in the current working directory
    darwin_dir = pathlib.Path("darwin")

    # If it exists, remove it to test that our code doesn't create it
    if darwin_dir.exists() and darwin_dir.is_dir():
        try:
            for item in darwin_dir.iterdir():
                if item.is_file():
                    item.unlink()
            darwin_dir.rmdir()
        except Exception:
            # If we can't remove it, skip this check
            pass

    # Verify the client's cache directory is not in the current working directory
    current_dir = pathlib.Path.cwd()
    assert darwin_client.html_cache is not None, "HTML cache should not be None"
    assert current_dir / "darwin" != darwin_client.html_cache.cache_dir

    # Check if the darwin directory was created in the current directory
    # This should not happen with our fix
    assert not darwin_dir.exists(), "The 'darwin' directory should not be created in the current directory"

    # Verify the cache directory is a properly structured path in the OS cache location
    assert darwin_client.html_cache is not None, "HTML cache should not be None"
    assert str(darwin_client.html_cache.cache_dir).startswith(str(get_default_cache_dir()))


@pytest.mark.asyncio
async def test_empty_dataset_validation():
    """Test that empty datasets are not cached."""
    # Create a client with a mock html_client
    html_client = MagicMock(spec=HTMLClient)
    html_client.cache = MagicMock(spec=HTMLCache)
    html_client.cache.set_data = MagicMock(return_value={"stored": True})
    html_client.cache.set_binary_data = MagicMock(return_value={"stored": True})

    darwin_client = DarwinClient(html_client=html_client)

    # Setup empty data
    darwin_client.options = {}
    darwin_client.total_options = 0
    darwin_client.total_categories = 0
    darwin_client.name_index = {}
    darwin_client.word_index = defaultdict(set)
    darwin_client.prefix_index = {}

    # Try to save an empty dataset
    result = await darwin_client._save_to_filesystem_cache()

    # Check that it returns False
    assert result is False

    # Check that set_data was not called
    html_client.cache.set_data.assert_not_called()
    html_client.cache.set_binary_data.assert_not_called()

    # Now test with a small but non-zero dataset
    darwin_client.options = {
        "test.option1": DarwinOption(name="test.option1", description="test description"),
        "test.option2": DarwinOption(name="test.option2", description="test description"),
    }
    darwin_client.total_options = 2
    darwin_client.total_categories = 1
    darwin_client.name_index = {"test": ["test.option1", "test.option2"]}
    darwin_client.word_index = defaultdict(set)
    darwin_client.word_index["test"].add("test.option1")
    darwin_client.word_index["test"].add("test.option2")
    darwin_client.prefix_index = {"test": ["test.option1", "test.option2"]}

    # Should still fail due to having fewer than 10 options
    result = await darwin_client._save_to_filesystem_cache()
    assert result is False


@pytest.mark.asyncio
async def test_reject_empty_dataset_load():
    """Test that empty datasets are not loaded from cache."""
    # Create a client with a mock html_client
    html_client = MagicMock(spec=HTMLClient)
    html_client.cache = MagicMock(spec=HTMLCache)

    # Mock cache to return empty dataset
    empty_data = {
        "options": {},
        "total_options": 0,
        "total_categories": 0,
        "last_updated": datetime.now().isoformat(),
        "timestamp": time.time(),
    }

    empty_binary_data = {
        "name_index": {},
        "word_index": {},
        "prefix_index": {},
    }

    html_client.cache.get_data = MagicMock(return_value=(empty_data, {"cache_hit": True}))
    html_client.cache.get_binary_data = MagicMock(return_value=(empty_binary_data, {"cache_hit": True}))

    darwin_client = DarwinClient(html_client=html_client)

    # Try to load empty dataset
    result = await darwin_client._load_from_filesystem_cache()

    # Should return False to indicate loading failed
    assert result is False

    # Options should still be empty
    assert darwin_client.options == {}


def test_legacy_cache_cleanup():
    """Test that legacy cache files in the current directory are cleaned up during invalidation."""
    # Use a temporary directory instead of the real darwin directory
    with tempfile.TemporaryDirectory() as temp_dir:
        # Create a mock legacy darwin directory in the temp directory
        darwin_dir = pathlib.Path(temp_dir) / "darwin"
        darwin_dir.mkdir()

        # Create dummy cache files
        (darwin_dir / "test.html").write_text("test html content")
        (darwin_dir / "test.data.json").write_text('{"test": "data"}')
        (darwin_dir / "test.data.pickle").write_bytes(b"test pickle data")

        # Verify the directory and files exist
        assert darwin_dir.exists()
        assert (darwin_dir / "test.html").exists()

        # Create a special patched version of the invalidate_cache method that works on our test directory
        # instead of the current directory
        with patch.object(DarwinClient, "invalidate_cache", autospec=True) as mock_invalidate:

            def custom_invalidate(self):
                # Custom version of invalidate_cache that uses our test directory
                # This simulates what the real method would do but in our test directory
                logger = logging.getLogger(__name__)
                logger.info("Mock invalidating nix-darwin data cache")

                # Use our test directory instead of pathlib.Path("darwin")
                legacy_bad_path = darwin_dir

                if legacy_bad_path.exists() and legacy_bad_path.is_dir():
                    logger.warning("Found legacy 'darwin' directory in test path - attempting cleanup")
                    try:
                        # Only remove if it has cache files
                        safe_to_remove = True
                        for item in legacy_bad_path.iterdir():
                            condition1 = item.name.endswith(".html")
                            condition2 = item.name.endswith(".data.json")
                            condition3 = item.name.endswith(".data.pickle")
                            if not (condition1 or condition2 or condition3):
                                safe_to_remove = False
                                break

                        if safe_to_remove:
                            for item in legacy_bad_path.iterdir():
                                if item.is_file():
                                    logger.info(f"Removing legacy cache file: {item}")
                                    item.unlink()
                            logger.info("Removing legacy darwin directory")
                            legacy_bad_path.rmdir()
                        else:
                            logger.warning("Legacy 'darwin' directory contains non-cache files - not removing")
                    except Exception as cleanup_err:
                        logger.warning(f"Failed to clean up legacy cache: {cleanup_err}")

                logger.info("Mock nix-darwin data cache invalidated")

            # Replace the mocked method with our custom implementation
            mock_invalidate.side_effect = custom_invalidate

            # Create a client
            html_client = HTMLClient()
            darwin_client = DarwinClient(html_client=html_client)

            # Call invalidate_cache which should clean up the legacy directory
            darwin_client.invalidate_cache()

            # Verify the directory was removed or the mock was called
            assert mock_invalidate.called

            # Either the directory should be gone or we should check if the cleanup would have worked
            if darwin_dir.exists():
                # The mock might not have actually removed the directory, but it should have been called
                mock_calls = [args[0] for args in mock_invalidate.call_args_list]
                assert darwin_client in mock_calls
